var fs    = require("fs");
var path  = require("path");
var less  = require("less");
var ejs   = require("ejs");
var utils = require("./utils.js");
var proc  = require("./process.js");

exports.builders = builders = {
	
	less : function(inputFilepath, outputFilepath) {
		fs.readFile(inputFilepath, "utf-8", function(error, data) {
			if (error) {
				proc.quit(error);
			};
			
			var dir = path.dirname(inputFilepath);
			var filename = path.basename(inputFilepath);
			var parser = new(less.Parser)({
				paths		: [dir],
				filename	: filename
			});
			
			parser.parse(data, function(error, tree) {
				if (error) {
					proc.quit(error);
				}
				
				outputFilepath = outputFilepath.replace(/\.less$/i, ".css");
				fs.writeFile(outputFilepath, tree.toCSS({compress : !global.config.debug}), function(error) {
					if (error) {
						proc.quit(error);
					};
					
					// console.log("CREATED: " + outputFilepath);
				});
			});
		});
	},
	
	ejs : function(inputFilepath, outputFilepath) {
		fs.readFile(inputFilepath, "utf-8", function(error, data) {
			if (error) {
				proc.quit(error);
			};
			
			outputFilepath = outputFilepath.replace(/\.ejs$/i, ".html");
			fs.writeFile(outputFilepath, ejs.render(data, {filename : inputFilepath}), function(error) {
				if (error) {
					proc.quit(error);
				};
				
				// console.log("CREATED: " + outputFilepath);
			});
		});
	},
	
	dir : function(dir) {
		dir && fs.mkdir(dir, function() {
			// console.log("CREATED: " + dir);
		});
	},
	
	other : function(inputFilepath, outputFilepath) {
		fs.readFile(inputFilepath, function(error, data) {
			if (error) {
				console.error(error);
				proc.quit();
			}
			
			fs.writeFile(outputFilepath, data, function() {
				// console.log("CREATED: " + outputFilepath);
			});
		});
	}
	
};

exports.build = function(inputDir, outputDir) {
	utils.emptyDir(outputDir);
	_build(inputDir, outputDir);
};

function _build(inputDir, outputDir) {
	utils.readDir(inputDir, function(dir, file, isDir) {
		var inputFilepath = dir + "/" + file;
		var outputFilepath = inputFilepath.replace(inputDir, outputDir);
		var ext = path.extname(inputFilepath).substr(1);
		
		if (isDir) {
			builders.dir(outputFilepath);
		}
		else {
			(builders[ext] || builders.other)(inputFilepath, outputFilepath);
		}
	}, /^_.*$/i);
}
